#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements the percentile bootstrap method described in (Wilcox, 2012) [pp.332-335]
# for comparing measures of location for two independent groups. In our case,
# the two groups are represented by the agreement matrixes A1 and A2.
#
# - FUN specifies the calculation function for the agreement rate, e.g., mean, median, etc.
# - weight is 0.5 for comparing medians; see (Wilcox, 2012) [p.335] and 1 otherwise.
# - B is the number of boostrap samples
#
# R. Wilcox. (2012). Modern Statistics for the Social and Behavioral Sciences. CRC Press
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
between.subjects.percentile.bootstrap.AR <- function(A1, A2, FUN = mean, weight = 1.0, B = 2000)
{
  N1 <- nrow(A1)
  N2 <- nrow(A2)
  
  A <- 0
  C <- 0
  for(trial in 1:B)
  {
    # generate two bootstrap samples
    indexes1 <- sample(1:N1, N1, replace = TRUE)
    indexes2 <- sample(1:N2, N2, replace = TRUE)
    
    # Is the estimate measure for the first group larger than the estimate for the second?
    diff <- FUN(ARs(A1[indexes1,indexes1])) - FUN(ARs(A2[indexes2,indexes2]))
    if (diff > 0) A <- A+1
    else if (diff == 0) C <- C+1
  }
  p <- (A + weight*C)/B
  p <- 2*min(p, 1-p)
  
  return (list(p.value = p))
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements the bootstrap-t method described in (Wilcox, 2012) [p.339]
# for comparing means for two independent groups. In our case,
# the two groups are represented by their agreement matrixes A1 and A2.
#
# - B is the number of boostrap samples
#
# R. Wilcox. (2012). Modern Statistics for the Social and Behavioral Sciences. CRC Press
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
between.subjects.bootstrap.t.AR <- function(A1, A2, B = 2000)
{
  N1 <- nrow(A1)
  N2 <- nrow(A2)
  
  # computes Welch's W statistic for the two groups
  ARs1 <- ARs(A1)
  ARs2 <- ARs(A2)
  diff <- mean(ARs1) - mean(ARs2)
  W <- diff / sqrt(sd(ARs1)^2/N1 + sd(ARs2)^2/N2)
  
  if (any(is.nan(W),is.infinite(W))) # safety check
    return (list(p.value = 0))
  
  # bootstrap Welch's W statistic
  b.W <- c()
  for(trial in 1:B)
  {
     # generate two bootstrap samples
     indexes1 <- sample(1:N1, N1, replace = TRUE)
     indexes2 <- sample(1:N2, N2, replace = TRUE)
      
     # compute the bootstrap estimate W
     b.ARs1 <- ARs(A1[indexes1,indexes1])
     b.ARs2 <- ARs(A2[indexes2,indexes2])
     b.W[[trial]] <- ((mean(b.ARs1) - mean(b.ARs2)) - diff) / sqrt(sd(b.ARs1)^2/N1 + sd(b.ARs2)^2/N2)
     if (any(is.nan(b.W[[trial]]),is.infinite(b.W[[trial]]))) # safety check
       b.W[[trial]] <- 0
  }
    
  # compute the 95% CI for the boostrapped W's
  q <- quantile(b.W, probs = c(0.025, 0.975))
    
  # reject if the original W value is outside the 95% CI
  p <- ifelse(any(W <= q[[1]], W >= q[[2]]), 0, 1) # not an actual p-value, but useful to automatically compute the Type I error rate in our simulations

  return (list(p.value = p))
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements a variant of the Vb test (Vatavu and Wobbrock, 2016) for between-subjects designs,
# updated to account for the non-transitivity of the agreement relation.
#
# The implementation is based on the original R code from Theophanis Tsandilas:
# http://https://github.com/ephanis/TOCHI-supplementary 
#
# Radu-Daniel Vatavu and Jacob O. Wobbrock. 2016. Between-Subjects Elicitation Studies: 
# Formalization and Tool Support. In Proceedings of the 2016 CHI Conference on Human Factors 
# in Computing Systems (CHI '16). ACM, New York, NY, USA, 3390-3402. 
# DOI: https://doi.org/10.1145/2858036.2858228
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Vb.test <- function(A1, A2, conditionalMargins = TRUE)
{
  Vb.Measure <- function(e1, e2, N1, N2)
  {
    expected1 <- N1*(e1+e2)/(N1+N2)
    expected2 <- N2*(e1+e2)/(N1+N2)
    return ((e1-expected1)^2 + (e2-expected2)^2)
  }
  
  N1 <- nrow(A1)
  N2 <- nrow(A2)
  n1 <- N1*(N1-1)/2
  n2 <- N2*(N2-1)/2
  a1 <- (sum(A1)-N1)/2
  a2 <- (sum(A2)-N2)/2
  
  V <- Vb.Measure(a1, a2, n1, n2)
  pObserved <- choose(n1,a1) * choose(n2,a2)
  
  total <- 0
  Pi <- 0
  for(e1 in 0:n1)
    for(e2 in 0:n2)
      if (any(all(conditionalMargins == TRUE, e1+e2 == a1+a2), conditionalMargins == FALSE))
      {
        p <- choose(n1,e1) * choose(n2,e2)
        total <- total + p
        
        if (Vb.Measure(e1, e2, n1, n2) >= V)
          Pi <- Pi + p
      }
  
  Pi <- ifelse(total == 0, 0, (Pi - 0.5*pObserved)/total)
  return (list(test = V, p.value = Pi, df = 2))
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements the original version of the Vb test (Vatavu and Wobbrock, 2016) for
# between-subjects designs (transitivity is assumed).
#
# The implementation is based on the original R code from Theophanis Tsandilas:
# http://https://github.com/ephanis/TOCHI-supplementary 
# (The R code was updated to compute the p-value under the conditional margins
# condition as well, i.e., when a1+a2 is fixed; see function PrConditionalMargins.)
#
# Radu-Daniel Vatavu and Jacob O. Wobbrock. 2016. Between-Subjects Elicitation Studies: 
# Formalization and Tool Support. In Proceedings of the 2016 CHI Conference on Human Factors 
# in Computing Systems (CHI '16). ACM, New York, NY, USA, 3390-3402. 
# DOI: https://doi.org/10.1145/2858036.2858228
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Vb.test.2016 <- function(A1, A2, conditionalMargins = TRUE)
{
  #Calculation of Vb for groups with pairs numbers n1 and n2 and agreement configurations e1 and e2
  Vb <- function(e1, e2, n1, n2){
    e = e1 + e2
    a = e / (n1 + n2)
    expected1 = n1*a
    expected2 = n2*a
    (e1 - expected1)^2 + (e2 - expected2)^2
  }
  
  # Calculation of probability of observing a1, a2 or more extreme proportions for groups with N1 and N2 participants
  Pr <- function(a1, a2, N1, N2){
    n1 <- N1 * (N1 - 1) /2
    n2 <- N2 * (N2 - 1) /2
    
    pr1 <- probabilities(N1)
    pr2 <- probabilities(N2)
    
    v_ <- Vb(a1, a2, n1, n2)
    
    p12 <- 0.5 * pr1[as.character(a1)]*pr2[as.character(a2)]
    if(is.na(p12)) p12 = 0
    
    for(e1 in as.integer(names(pr1))){
      for(e2 in as.integer(names(pr2))){
        v <- Vb(e1, e2, n1, n2)
        
        if(v > v_){
          p <- pr1[as.character(e1)]*pr2[as.character(e2)]
          p12 <- p12 + p
        }
        
      }
    }
    
    p12
  }
  
  # Conditional margins version of Pr
  PrConditionalMargins <- function(a1, a2, N1, N2){
    n1 <- N1 * (N1 - 1) /2
    n2 <- N2 * (N2 - 1) /2
    
    pr1 <- probabilities(N1)
    pr2 <- probabilities(N2)
    
    v_ <- Vb(a1, a2, n1, n2)
    
    p12 <- 0.5 * pr1[as.character(a1)]*pr2[as.character(a2)]
    if(is.na(p12)) p12 = 0
    
    total <- p12
    for(e1 in as.integer(names(pr1)))
      for(e2 in as.integer(names(pr2)))
        if (e1+e2 == a1+a2)
        {
          v <- Vb(e1, e2, n1, n2)
          p <- pr1[as.character(e1)]*pr2[as.character(e2)]
          
          total <- total + p 
          if(v > v_)
            p12 <- p12 + p
        }
    
    return (ifelse(total == 0, 0, p12/total))
  }
  
  # Calculate the probability of each possible number of agreeing pairs for N participants
  probabilities <- function(N){
    partitions <- as.matrix(parts(N))
    agreements <- partitions.agreements(partitions)
    freq <- partitions.frequencies(partitions)
    frame <- data.frame(agreements=agreements, f=freq)
    with(frame, tapply(f, agreements, FUN = sum)) / sum(freq)
  }
  
  # Calculate the number of agreeing pairs for all partitions
  partitions.agreements <- function(partitions){
    m <- partitions*(partitions - 1) / 2
    colSums(m)
  }
  
  # Calculate the frequency of observing partitions
  partitions.frequencies <- function(partitions){
    npartitions <- length(partitions[1,])
    nparticipants <- length(partitions[,1])
    
    f <- rep(1, npartitions)
    
    #print(partitions)
    
    for(i in 1:npartitions){
      partition <- partitions[,i]
      
      n <- nparticipants
      for(j in 1:nparticipants){
        f[i] <- f[i]*choose(n, partition[j])
        n <- n - partition[j]
      }
      
      t <- table(partition)
      t <- t[names(t) != '0'] # Do not conider repititions of 0
      t <- t[t > 1]
      
      if(length(t) == 0) denom <- 1
      else denom <- prod(factorial(t))
      
      f[i] <- f[i] / denom
    }
    
    f
  }
  
  N1 <- nrow(A1)
  N2 <- nrow(A2)
  n1 <- N1*(N1-1)/2
  n2 <- N2*(N2-1)/2
  a1 <- (sum(A1)-N1)/2
  a2 <- (sum(A2)-N2)/2
  
  return(list(
    test = Vb(a1, a2, n1, n2), 
    p.value = ifelse(conditionalMargins == FALSE, Pr(a1,a2,N1,N2), PrConditionalMargins(a1,a2,N1,N2)),
    df = 2
  ))
}