#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Runs Type I and Power simulations for between-subjects and within-subjects designs
# (These simulations take a long time to run, approx. 24 hours for N=20/REPETITIONS=100.)
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
runTypeIandPowerSimulations <- function(
  POPULATION_SIZE = 100,  # size of the population used to estimate the values of epsilon's for various target AR's
  N = 20,                 # sample size
  REPETITIONS = 100,      # number of samples of size N drawn from the infinite population
  SEED = 1000)            # random numbers seed for replication purposes
{
  print("Between-subjects Type I error rate") 
  exp <- simulations.BetweenSubjects.TypeI(POPULATION_SIZE, N, REPETITIONS, SEED)
  write.csv(exp, file = "between-subjects-TypeIError.csv", row.names = FALSE, col.names = TRUE, sep = ",")
  
  print("Between-subjects Power")
  exp <- simulations.BetweenSubjects.Power(POPULATION_SIZE, N, REPETITIONS, SEED)
  write.csv(exp, file = "between-subjects-Power.csv", row.names = FALSE, col.names = TRUE, sep = ",")
  
  print("Within-subjects Type I error rate")
  exp <- simulations.WithinSubjects.TypeI(POPULATION_SIZE, N, REPETITIONS, SEED)
  write.csv(exp, file = "within-subjects-TypeIError.csv", row.names = FALSE, col.names = TRUE, sep = ",")
  
  print("Within-subjects Power")
  exp <- simulations.WithinSubjects.Power(POPULATION_SIZE, N, REPETITIONS, SEED)
  write.csv(exp, file = "within-subjects-Power.csv", row.names = FALSE, col.names = TRUE, sep = ",")
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
## Generates a Latex table (summary) for p values
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
LatexTable.Summary <- function(data, TESTS)
{
  numTrials <- 0
  for(i in 1:length(TESTS))
  {
    p <- data[data$Test == TESTS[[i]],]$p
    p <- p[!is.na(p)]
    
    TypeI05 <- length(p[p<=.05])/length(p)
    TypeI01 <- length(p[p<=.01])/length(p)
    numTrials <- numTrials + length(p)
    
    cat(sprintf("%15s", TESTS[[i]]), "&", sprintf("%7.3f", mean(TypeI05)), "&", sprintf("%7.3f", mean(TypeI01)), "\r\n")
  }
  cat("Results reported from", numTrials, "trials.\r\n")
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
## Generates Latex table (detailed) for p values
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
LatexTable.Detailed <- function(data, TESTS, report05 = TRUE)
{
  numTrials <- 0
  print(ddply(data, ~k+Model, function(dataframe)
  {
    t05 <- c()
    t01 <- c()
    for(i in 1:length(TESTS))
    {
      p <- dataframe[dataframe$Test == TESTS[[i]],]$p
      p <- p[!is.na(p)]
      t05 <- c(t05, length(p[p<=.05])/length(p))
      t01 <- c(t01, length(p[p<=.01])/length(p))
      numTrials <<- numTrials + length(p)
    }
    t05 <- sprintf("& %7.3f", t05)
    t01 <- sprintf("& %7.3f", t01)
    names(t05) <- TESTS
    names(t01) <- TESTS
    
    if (report05) return (c(t05, "\\"))
    else return (c(t01, "\\"))
  }))
  cat("Results reported from", numTrials, "trials.\r\n")
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
## statistical inference tests for comparing agreement rates (AR) for between-subjects designs
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
saferun <- function(fun) return (tryCatch(fun, error = function(e) { return (list(p.value = NA)) }))

bs.ttest <- function(A1,A2) return (saferun(t.test(ARs(A1), ARs(A2), paired = FALSE, var.equal = TRUE)))
bs.Welch <- function(A1,A2) return (saferun(t.test(ARs(A1), ARs(A2), paired = FALSE, var.equal = FALSE)))
bs.wmw    <- function(A1,A2) return (saferun(wmw(ARs(A1), ARs(A2))))
bs.cliff  <- function(A1,A2) return (saferun(cidv2(ARs(A1), ARs(A2), plotit = FALSE)))
bs.bmp    <- function(A1,A2) return (saferun(bmp(ARs(A1), ARs(A2))))
bs.ks     <- function(A1,A2) return (saferun(ks(ARs(A1), ARs(A2))))
Vb.test.2016.CM  <- function(A1,A2) return (saferun(Vb.test.2016(A1, A2, conditionalMargins = TRUE)))
Vb.test.2016.nCM <- function(A1,A2) return (saferun(Vb.test.2016(A1, A2, conditionalMargins = FALSE)))
bspb.mean    <- function(A1,A2) return (saferun(between.subjects.percentile.bootstrap.AR(A1, A2, FUN = mean)))
bsbt.mean    <- function(A1,A2) return (saferun(between.subjects.bootstrap.t.AR(A1, A2)))

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
## statistical inference tests for comparing agreement rates AR for within-subjects designs
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
ws.ttest     <- function(A1,A2) return (saferun(t.test(ARs(A1), ARs(A2), paired = TRUE)))
ws.wilcoxon  <- function(A1,A2) return (saferun(wilcox.test(ARs(A1), ARs(A2), paired = TRUE)))
vrd2015      <- function(A1,A2) return (saferun(Vrd.test.2015(A1, A2)))
vrd          <- function(A1,A2) return (saferun(Vrd.test(A1, A2)))
wspb.mean    <- function(A1,A2) return (saferun(within.subjects.percentile.bootstrap.AR(A1, A2, FUN = mean)))
wsbt.mean    <- function(A1,A2) return (saferun(within.subjects.bootstrap.t.AR(A1, A2)))

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
## Type I error rate simulations for between-subjects designs
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
simulations.BetweenSubjects.TypeI <- function(
  POPULATION_SIZE = 100,   # size of the population used to estimate the values of epsilon's for various target AR's
  N = 20,                  # sample size
  REPETITIONS = 100,       # number of samples of size N drawn from the infinite population
  SEED)                    # random numbers seed for replication purposes
{
  # simulation set-up
  {
    # set seed for replication purposes
    if (!is.null(SEED)) 
    {
      set.seed(SEED)
      cat("Random number generator seed set to", SEED, "\r\n")
    }
    
    SDs <- c(5.42, 2.56, 1.63, 1.15, .88, .69, .576, .493, .416)  # SD parameters for the normal distributions (adopted from T. Tsandilas)
    betas <- c(.306, .64, 1.05, 1.56, 2.25, 3.26, 5.0, 8.3, 18.4) # B parameters for the Zipf-Mandelbrot distributions (adopted from T. Tsandilas)
    
    # textual descriptions of model + distribution + distance combinations
    Models <- c("Expert / Zipf-Mandelbrot / Identity",
                "Codebook / Zipf-Mandelbrot / Lin", 
                "Codebook / Bernoulli / Lin", 
                "Computer / Normal / Minkowski (p=0.5)", 
                "Computer / Normal / Minkowski (p=1)", 
                "Computer / Normal / Minkowski (p=2)", 
                "Computer / Normal / Squared Euclidean")
    
    # number of features to represent end-users' proposals
    Ks <- c(5, 10)
    
    # generator functions for dissimilarity matrices
    FUNs <- c(generateDM.ZipfMandelbrot.Overlap, 
              generateDM.ZipfMandelbrot.Lin, 
              generateDM.Bernoulli.Lin, 
              generateDM.Minkowski.0.5, 
              generateDM.Minkowski.1, 
              generateDM.Minkowski.2, 
              generateDM.SqrEuclidean)
    
    # statistical inference tests
    TESTS <- list(
      list(Name = "t-test", Fun = bs.ttest),
      list(Name = "Welch's t-test", Fun = bs.Welch),
      list(Name = "Mann-Whitney", Fun = bs.wmw),
      list(Name = "Cliff", Fun = bs.cliff),
      list(Name = "BMP", Fun = bs.bmp),
      list(Name = "KS", Fun = bs.ks),
      list(Name = "Vb2016-CM", Fun = Vb.test.2016.CM),
      list(Name = "Vb2016-nCM", Fun = Vb.test.2016.nCM),
      list(Name = "pb-m", Fun = bspb.mean),
      list(Name = "bt", Fun = bsbt.mean)
    )
  }
  
  # dataframe with results returned from this function
  exp <- NULL
  
  # Run simulations for the Zipf-Mandelbrot (ZM) distribution and the identity distance (overlap with k=1) 
  cat(Models[[1]], "& 1 \r\n")
  for(i in 1:length(betas))
  {
    results <- between.subjects.TypeI(POPULATION_SIZE = POPULATION_SIZE, 
                                      N = N, 
                                      REPETITIONS = REPETITIONS, 
                                      TARGET_ARS = c(0),  # the value betas[i] below is the only information necessary to generate target AR's for the ZM distribution
                                      TESTS = TESTS,
                                      generateDM = FUNs[[1]], params = c(betas[[i]]),
                                      VERBOSE = FALSE
    )
    results$TargetAR <- i * 0.1   # fill in the target AR
    results$k <- 1                # fill in the number of features: one feature for this simulation (i.e., the final sign)
    results$Model <- Models[[1]]  # fill in the model
    exp <- rbind(exp, results)    # bind results to dataframe
  }
  
  # run simulations for the rest of the models
  for(k in Ks) 
    for(i in 2:length(Models)) 
    {
      cat(Models[[i]], "&", k, "\r\n")
      
      for(repetition in 1:REPETITIONS)
      { 
        cat("... repetition", repetition, "of", REPETITIONS, "\r\n")
        
        if (i == 2) params <- sample(betas, k, replace = TRUE)  # Zipf-Mandelbrot
        else if (i == 3) params <- runif(k, 0, 1)               # Bernoulli
        else params <- sample(SDs, k, replace = TRUE)           # Normal
        
        results <- between.subjects.TypeI(POPULATION_SIZE = POPULATION_SIZE,
                                          N = N,
                                          REPETITIONS = 1,  # set to 1 here because the number of repetitions is controlled in the for loop to make sure that different params are used for each repetition
                                          TARGET_ARS = seq(0.1, 0.9, 0.1),
                                          TESTS = TESTS, 
                                          generateDM = FUNs[[i]], params = params, 
                                          VERBOSE = FALSE
        )
        results$Trial <- repetition   # fill in the trial number
        results$k <- k                # fill in the number of features
        results$Model <- Models[[i]]  # fill in the model
        exp <- rbind(exp, results)    # bind results to dataframe
      }
    }
  
  exp$Model <- as.factor(exp$Model)
  return (exp)
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements a simulation procedure to estimate the Type I error rate for 
# AR statistical tests (provided in the TESTS array) for between-subjects experimental designs.
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
between.subjects.TypeI <- function(
  POPULATION_SIZE = 100,            # size of the population used to estimate the values of epsilon's for various target AR's
  N = 20,                           # sample size
  REPETITIONS = 100,                # number of samples of size N drawn from the infinite population
  TARGET_ARS = seq(0.1, 0.9, 0.1),  # target agreement rates
  TESTS,                            # array of functions that implement statistical inference tests for comparing AR's
  generateDM, params,               # *generateDM* is a function that generates dissimilarity matrices parameterized by *params*
  VERBOSE = FALSE)                  # if TRUE, intermediate results (progress) are listed at the console
{
  # dataframe with results returned from this function
  results <- list() 
  
  # run the simulation for a total number of trials = REPETITIONS x length(TARGET_ARS)
  for(trial in 1:REPETITIONS)
  {
    DM <- generateDM(POPULATION_SIZE, params)$DM         # generate a population of finite size
    epsilon <- Epsilon.fromDM.ExactARs(DM, TARGET_ARS)   # use the population to compute epsilon's for specified target AR's as estimates of the epsilon's of the infinite population
    
    for(j in 1:length(epsilon))
    {
      # sample two independent groups from the infinite population and compute agreement matrices
      proposals <- generateDM(2*N, params, NULL, GENERATE_PROPOSALS_ONLY = TRUE)$values
      group1 <- as.matrix(proposals[1:N,])
      group2 <- as.matrix(proposals[(N+1):(2*N),])
      A1 <- AM.fromDM(generateDM(N, params, group1)$DM, epsilon[[j]])
      A2 <- AM.fromDM(generateDM(N, params, group2)$DM, epsilon[[j]])

      # run tests
      for(i in 1:length(TESTS)) 
        results <- rbind(results, list(N, TARGET_ARS[[j]], trial, AR(A1), AR(A2), TESTS[[i]]$Name, TESTS[[i]]$Fun(A1,A2)$p.value))
      
      # if verbose, print results to the console
      if (VERBOSE) cat(sprintf("%3d. AR1=%.3f AR2=%.3f \r\n", trial, AR(A1), AR(A2)))
    }
  }
  
  # create data frame from the list of results
  results <- as.data.frame(results)
  names(results) <- c("N", "TargetAR", "Trial", "AR1", "AR2", "Test", "p")
  results$N <- as.integer(results$N)
  results$TargetAR <- as.numeric(results$TargetAR)
  results$Trial <- as.integer(results$Trial)
  results$AR1 <- as.numeric(results$AR1)
  results$AR2 <- as.numeric(results$AR2)
  results$Test <- as.factor(as.character(results$Test))
  results$p <- as.numeric(results$p)
  
  return (results)
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
## Power simulations for between-subjects designs
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
simulations.BetweenSubjects.Power <- function(
  POPULATION_SIZE = 100,   # size of the population used to estimate the values of epsilon's for various target AR's
  N = 20,                  # sample size
  REPETITIONS = 100,       # number of samples of size N drawn from the infinite population
  SEED)                    # random numbers seed for replication purposes
{
  # simulation set-up
  {
    # set seed for replication purposes
    if (!is.null(SEED)) 
    {
      set.seed(SEED)
      cat("Random number generator seed set to", SEED, "\r\n")
    }
    
    SDs <- c(5.42, 2.56, 1.63, 1.15, .88, .69, .576, .493, .416)  # SD parameters for the normal distributions (adopted from T. Tsandilas)
    betas <- c(.306, .64, 1.05, 1.56, 2.25, 3.26, 5.0, 8.3, 18.4) # B parameters for the Zipf-Mandelbrot distributions (adopted from T. Tsandilas)
    
    # textual descriptions of model + distribution + distance combinations
    Models <- c("Expert / Zipf-Mandelbrot / Identity",
                "Codebook / Zipf-Mandelbrot / Lin", 
                "Codebook / Bernoulli / Lin", 
                "Computer / Normal / Minkowski (p=0.5)", 
                "Computer / Normal / Minkowski (p=1)", 
                "Computer / Normal / Minkowski (p=2)", 
                "Computer / Normal / Squared Euclidean")
    
    # number of features to represent end-users' proposals
    Ks <- c(5, 10)
    
    # generator functions
    FUNs <- c(generateDM.ZipfMandelbrot.Overlap, 
              generateDM.ZipfMandelbrot.Lin, 
              generateDM.Bernoulli.Lin, 
              generateDM.Minkowski.0.5, 
              generateDM.Minkowski.1, 
              generateDM.Minkowski.2, 
              generateDM.SqrEuclidean)
    
    # statistical inference tests
    TESTS <- list(
      list(Name = "t-test", Fun = bs.ttest),
      list(Name = "Welch's t-test", Fun = bs.Welch),
      list(Name = "Mann-Whitney", Fun = bs.wmw),
      list(Name = "Cliff", Fun = bs.cliff),
      list(Name = "BMP", Fun = bs.bmp),
      list(Name = "KS", Fun = bs.ks),
      list(Name = "Vb2016-CM", Fun = Vb.test.2016.CM),
      list(Name = "Vb2016-nCM", Fun = Vb.test.2016.nCM),
      list(Name = "pb-m", Fun = bspb.mean),
      list(Name = "bt", Fun = bsbt.mean)
    )
  }
  
  # dataframe with results returned from this function
  exp <- NULL
  
  # Run simulations for the Zipf-Mandelbrot (ZM) distribution and the identity distance (overlap with k=1)
  cat(Models[[1]], "& 1 \r\n")
  for(i in 1:(length(betas)-1))
    for(j in (i+1):length(betas))
    {
      results <- between.subjects.Power(POPULATION_SIZE = POPULATION_SIZE,
                                        N = N, 
                                        REPETITIONS = REPETITIONS, 
                                        TARGET_ARS = list(c(0,0)), # values betas[i] and betas[j] (provided below) are the only information necessary to generate the target AR's for the ZM distribution
                                        TESTS = TESTS, 
                                        generateDM = FUNs[[1]], params1 = c(betas[[i]]), params2 = c(betas[[j]]), 
                                        VERBOSE = FALSE
      )
      results$TargetAR1 <- i * 0.1  # fill in target AR for the first population
      results$TargetAR2 <- j * 0.1  # fill in target AR for the second population
      results$k <- 1             
      results$Model <- Models[[1]]  # fill in the model
      exp <- rbind(exp, results)    # bind results to dataframe
    }
  
  # run simulations for the rest of the models
  TARGET_ARS <- list()
  trial <- 1
  for(TARGET_AR_1 in seq(0.1, 0.8, 0.1))
    for(TARGET_AR_2 in seq(TARGET_AR_1 + 0.1, 0.9, 0.1)) 
    {
      TARGET_ARS[[trial]] <- c(TARGET_AR_1, TARGET_AR_2)
      trial <- trial + 1
    }
  
  for(k in Ks) 
    for(i in 2:length(Models)) 
    {
      cat(Models[[i]], "&", k, "\r\n")
      
      for(repetition in 1:REPETITIONS)
      {
        cat("... repetition", repetition, "of", REPETITIONS, "\r\n")
        
        if (i == 2) params <- sample(betas, k, replace = TRUE)  # Zipf-Mandelbrot
        else if (i == 3) params <- runif(k, 0, 1)               # Bernoulli
        else params <- sample(SDs, k, replace = TRUE)           # Normal
        
        results <- between.subjects.Power(POPULATION_SIZE = POPULATION_SIZE, 
                                          N = N, 
                                          REPETITIONS = 1, # set to 1 here because the number of repetitions is controlled in the for loop to make sure that different params are used for each repetition
                                          TARGET_ARS = TARGET_ARS, 
                                          TESTS = TESTS, 
                                          generateDM = FUNs[[i]], params1 = params, params2 = params, 
                                          VERBOSE = FALSE
        ) 
        results$Trial <- repetition   # fill in trial information
        results$k <- k                # fill in the number of features
        results$Model <- Models[[i]]  # fill in the model
        exp <- rbind(exp, results)    # bind results to dataframe
      }
    }
  
  exp$Model <- as.factor(exp$Model)
  return (exp)
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements a simulation procedure to estimate the power of AR statistical tests 
# and between-subjects designs.
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
between.subjects.Power <- function(
  POPULATION_SIZE = 100,         # size of the population used to estimate the values of epsilon's for various target AR's
  N = 20,                        # sample size
  REPETITIONS = 100,             # number of samples of size N drawn from the (infinite) population
  TARGET_ARS,                    # target agreement rates for which populations are generated
  TESTS,                         # array of functions that implement statistical inference tests for comparing ARs
  generateDM, params1, params2,  # *generateDM* is a function that generates dissimilarity matrices parameterized by *params1* (for the first population) and *params2* (for the second population)
  VERBOSE = FALSE)               # if TRUE, intermediate results (progress) are listed at the console
{
  # dataframe with results returned from this function
  results <- list()
  
  # run simulation for a total number of trials = REPETITIONS x length(TARGET_ARS)
  for(trial in 1:REPETITIONS)
  {
    DM1 <- generateDM(POPULATION_SIZE, params1)$DM   # generate a population of finite size
    DM2 <- generateDM(POPULATION_SIZE, params2)$DM   # generate a second population of finite size
    epsilon1 <- Epsilon.fromDM.ExactARs(DM1, sapply(TARGET_ARS, FUN=function(x) return (x[[1]])))  # use the first population to estimate epsilons for specified target AR's
    epsilon2 <- Epsilon.fromDM.ExactARs(DM2, sapply(TARGET_ARS, FUN=function(x) return (x[[2]])))  # use the second population to estimate epsilons for specified target AR's
    
    for(j in 1:length(epsilon1))
    {
      TARGET_AR_1 <- TARGET_ARS[[j]][[1]]
      TARGET_AR_2 <- TARGET_ARS[[j]][[2]]

      # sample two independent groups of size N from the two infinite populations
      A1 <- AM.fromDM(generateDM(N, params1)$DM, epsilon1[[j]])
      A2 <- AM.fromDM(generateDM(N, params2)$DM, epsilon2[[j]])
      
      # run tests
      for(i in 1:length(TESTS))
        results <- rbind(results, list(N, trial, TARGET_AR_1, TARGET_AR_2, AR(A1), AR(A2), TESTS[[i]]$Name, TESTS[[i]]$Fun(A1,A2)$p.value))
      
      # if verbose, print results to the console
      if (VERBOSE) cat(sprintf("%3d. AR1=%.3f AR2=%.3f DELTA=%.3f \r\n", trial, TARGET_AR_1, TARGET_AR_2, abs(TARGET_AR_1 - TARGET_AR_2)))
    }
  }
  
  # create data frame with the results
  results <- as.data.frame(results)
  names(results) <- c("N", "Trial", "TargetAR1", "TargetAR2", "AR1", "AR2", "Test", "p")
  results$N <- as.integer(results$N)
  results$Trial <- as.integer(results$Trial)
  results$TargetAR1 <- as.numeric(results$TargetAR1)
  results$TargetAR2 <- as.numeric(results$TargetAR2)
  results$AR1 <- as.numeric(results$AR1)
  results$AR2 <- as.numeric(results$AR2)
  results$Test <- as.factor(as.character(results$Test))
  results$p <- as.numeric(results$p)
  
  return (results)
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
## Type I error rate simulations for within-subjects designs
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
simulations.WithinSubjects.TypeI <- function(
  POPULATION_SIZE = 100,   # size of the population used to estimate the values of epsilon's for various target AR's
  N = 20,                  # sample size
  REPETITIONS = 100,       # number of samples of size N
  SEED)                    # random numbers seed for replication purposes
{
  # simulation set-up
  {
    # set seed for replication purposes
    if (!is.null(SEED)) 
    {
      set.seed(SEED)
      cat("Random number generator seed set to", SEED, "\r\n")
    }
    
    SDs <- c(5.42, 2.56, 1.63, 1.15, .88, .69, .576, .493, .416)  # SD parameters for the normal distributions (adopted from T. Tsandilas)
    betas <- c(.306, .64, 1.05, 1.56, 2.25, 3.26, 5.0, 8.3, 18.4) # B parameters for the Zipf-Mandelbrot distributions (adopted from T. Tsandilas)
    
    # textual descriptions of model + distribution + distance combinations
    Models <- c("Expert / Zipf-Mandelbrot / Identity",
                "Codebook / Zipf-Mandelbrot / Lin", 
                "Codebook / Bernoulli / Lin", 
                "Computer / Normal / Minkowski (p=0.5)", 
                "Computer / Normal / Minkowski (p=1)", 
                "Computer / Normal / Minkowski (p=2)", 
                "Computer / Normal / Squared Euclidean")
    
    # number of features to represent end-users' proposals
    Ks <- c(5, 10)
    
    # generator functions
    FUNs <- c(generateDM.ZipfMandelbrot.Overlap, 
              generateDM.ZipfMandelbrot.Lin, 
              generateDM.Bernoulli.Lin, 
              generateDM.Minkowski.0.5, 
              generateDM.Minkowski.1, 
              generateDM.Minkowski.2, 
              generateDM.SqrEuclidean)
    
    # statistical inference tests
    TESTS <- list(
      list(Name = "t-test", Fun = ws.ttest),
      list(Name = "Wilcoxon", Fun = ws.wilcoxon),
      list(Name = "Vrd2015", Fun = vrd2015),
      list(Name = "Vrd", Fun = vrd),
      list(Name = "pb-m", Fun = wspb.mean),
      list(Name = "bt", Fun = wsbt.mean)
    )
  }
  
  # dataframe with results returned by this function
  exp <- NULL
  
  # Run simulations for the ZM distribution and the identity distance (overlap with k=1) 
  cat(Models[[1]], "& 1 \r\n")
  for(i in 1:length(betas))
  {
    results <- within.subjects.TypeI(POPULATION_SIZE = POPULATION_SIZE, 
                                     N = N,
                                     REPETITIONS = REPETITIONS, 
                                     TARGET_ARS = c(0), # the value betas[i] below is the only information necessary to generate target AR's
                                     TESTS = TESTS, 
                                     generateDM = FUNs[[1]], params = c(betas[[i]]), 
                                     VERBOSE = FALSE
    )
    results$TargetAR <- i * 0.1    # fill in the target AR
    results$k <- 1                 # one feature for this simulation (i.e., the final sign)
    results$Model <- Models[[1]]   # fill in the model
    exp <- rbind(exp, results)     # bind results to dataframe
  }
  
  # run simulations for the rest of the models
  for(k in Ks) 
    for(i in 2:length(Models)) 
    {
      cat(Models[[i]], "&", k, "\r\n")
      
      for(repetition in 1:REPETITIONS)
      {
        cat("... repetition", repetition, "of", REPETITIONS, "\r\n")
        
        if (i == 2) params <- sample(betas, k, replace = TRUE) # Zipf-Mandelbrot
        else if (i == 3) params <- runif(k, 0, 1)              # Bernoulli
        else params <- sample(SDs, k, replace = TRUE)          # Normal distributions
        
        results <- within.subjects.TypeI(POPULATION_SIZE = POPULATION_SIZE, 
                                         N = N,
                                         REPETITIONS = 1,
                                         TARGET_ARS = seq(0.1, 0.9, 0.1),
                                         TESTS = TESTS,
                                         generateDM = FUNs[[i]], params = params, 
                                         VERBOSE = FALSE
        )
        results$Trial <- repetition   # fill in the trial number
        results$k <- k                # fill in the target AR    
        results$Model <- Models[[i]]  # fill in the model
        exp <- rbind(exp, results)    # bind results to dataframe
      }
    }
  
  exp$Model <- as.factor(exp$Model)
  return (exp)
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements a simulation procedure to estimate the Type I error rate for 
# AR statistical tests for within-subjects designs.
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
within.subjects.TypeI <- function(
  POPULATION_SIZE = 100,  # size of the population (of referents, not proposals) used to estimate the values of epsilon's for various target AR's
  N = 20,                 # sample size (proposals)
  REPETITIONS = 100,      # number of samples drawn repeatedly from the infinite population
  TARGET_ARS,             # array of target AR values for the populations of referents
  TESTS,                  # array of functions that implement statistical inference tests for comparing AR's
  generateDM, params,     # *generateDM* is a function that generates dissimilarity matrices parameterized by *params*
  VERBOSE = FALSE)        # if TRUE, progress results are printed at the console
{
  # dataframe with results returned by this function
  results <- list() 
  
  # run the simulation with total number of trials = REPETITIONS * length(TARGET_ARS)
  for(trial in 1:REPETITIONS)
  {
    epsilon <- generatePopulationOfReferentsOfExactAR(POPULATION_SIZE, N, generateDM, params, TARGET_ARS) # generate populations of finite size to estimate epsilon's for the infinite population
    
    for(j in 1:length(epsilon))
    {
      # sample two groups from the infinite population and compute agreement matrices
      proposals <- generateDM(2*N, params, NULL, GENERATE_PROPOSALS_ONLY = TRUE)$values
      group1 <- as.matrix(proposals[1:N,])
      group2 <- as.matrix(proposals[(N+1):(2*N),])
      A1 <- AM.fromDM(generateDM(N, params, group1)$DM, epsilon[[j]])
      A2 <- AM.fromDM(generateDM(N, params, group2)$DM, epsilon[[j]])
      
      # run statistical tests
      for(i in 1:length(TESTS)) 
        results <- rbind(results, list(N, TARGET_ARS[[j]], trial, AR(A1), AR(A2), TESTS[[i]]$Name, TESTS[[i]]$Fun(A1,A2)$p.value))
      
      # if verbose, print results to the console
      if (VERBOSE) cat(sprintf("%3d. AR1=%.3f AR2=%.3f \r\n", trial, AR(A1), AR(A2)))
    }
  }
  
  # create data frame from the list of results
  results <- as.data.frame(results)
  names(results) <- c("N", "TargetAR", "Trial", "AR1", "AR2", "Test", "p")
  results$N <- as.integer(results$N)
  results$TargetAR <- as.numeric(results$TargetAR)
  results$Trial <- as.integer(results$Trial)
  results$AR1 <- as.numeric(results$AR1)
  results$AR2 <- as.numeric(results$AR2)
  results$Test <- as.factor(as.character(results$Test))
  results$p <- as.numeric(results$p)
  
  return (results)
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
## Power simulations for within-subjects designs
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
simulations.WithinSubjects.Power <- function(
  POPULATION_SIZE = 100,   # size of the population used to estimate the values of epsilon's for various target AR's
  N = 20,                  # sample size
  REPETITIONS = 100,       # number of samples of size N
  SEED)                    # random numbers seed for replication purposes
{
  # simulation set-up
  {
    # set seed for replication purposes
    if (!is.null(SEED)) 
    {
      set.seed(SEED)
      cat("Random number generator seed set to", SEED, "\r\n")
    }
    
    SDs <- c(5.42, 2.56, 1.63, 1.15, .88, .69, .576, .493, .416)  # SD parameters for the normal distributions (adopted from T. Tsandilas)
    betas <- c(.306, .64, 1.05, 1.56, 2.25, 3.26, 5.0, 8.3, 18.4) # B parameters for the Zipf-Mandelbrot distributions (adopted from T. Tsandilas)
    
    # textual descriptions of model + distribution + distance combinations
    Models <- c("Expert / Zipf-Mandelbrot / Identity",
                "Codebook / Zipf-Mandelbrot / Lin", 
                "Codebook / Bernoulli / Lin", 
                "Computer / Normal / Minkowski (p=0.5)", 
                "Computer / Normal / Minkowski (p=1)", 
                "Computer / Normal / Minkowski (p=2)", 
                "Computer / Normal / Squared Euclidean")
    
    # number of features to represent end-users' proposals
    Ks <- c(5, 10)
    
    # generator functions
    FUNs <- c(generateDM.ZipfMandelbrot.Overlap, 
              generateDM.ZipfMandelbrot.Lin, 
              generateDM.Bernoulli.Lin, 
              generateDM.Minkowski.0.5, 
              generateDM.Minkowski.1, 
              generateDM.Minkowski.2, 
              generateDM.SqrEuclidean)
    
    # statistical inference tests
    TESTS <- list(
      list(Name = "t-test", Fun = ws.ttest),
      list(Name = "Wilcoxon", Fun = ws.wilcoxon),
      list(Name = "Vrd2015", Fun = vrd2015),
      list(Name = "Vrd", Fun = vrd),
      list(Name = "pb-m", Fun = wspb.mean),
      list(Name = "bt", Fun = wsbt.mean)
    )
  }
  
  # dataframe with results returned by this function
  exp <- NULL
  
  # Run simulations for the ZM distribution and the identity distance (overlap with k=1) 
  cat(Models[[1]], "& 1 \r\n")
  for(i in 1:(length(betas)-1))
    for(j in (i+1):length(betas))
    {
      results <- within.subjects.Power(POPULATION_SIZE = POPULATION_SIZE,
                                       N = N,
                                       REPETITIONS = REPETITIONS,
                                       TARGET_ARS = list(c(0,0)), # values betas[i] and betas[j] (provided below) are the only information necessary to generate the target AR's
                                       TESTS = TESTS, 
                                       generateDM = FUNs[[1]], params1 = c(betas[[i]]), params2 = c(betas[[j]]), 
                                       VERBOSE = FALSE
      )
      results$TargetAR1 <- i * 0.1  # fill in the target AR for the first population   
      results$TargetAR2 <- j * 0.1  # fill in the target AR for the second population
      results$k <- 1                # one feature for this simulation (i.e., the final sign) 
      results$Model <- Models[[1]]  # fill in the model
      exp <- rbind(exp, results)    # bind results to dataframe
    }
  
  # run simulations for the rest of the models
  TARGET_ARS <- list()
  trial <- 1
  for(TARGET_AR_1 in seq(0.1, 0.8, 0.1))
    for(TARGET_AR_2 in seq(TARGET_AR_1 + 0.1, 0.9, 0.1)) 
    {
      TARGET_ARS[[trial]] <- c(TARGET_AR_1, TARGET_AR_2)
      trial <- trial + 1
    }
  
  for(k in Ks) 
    for(i in 2:length(Models)) 
    {
      cat(Models[[i]], "&", k, "\r\n")
      
      for(repetition in 1:REPETITIONS)
      {
        cat("... repetition", repetition, "of", REPETITIONS, "\r\n")
        
        if (i == 2) params <- sample(betas, k, replace = TRUE) # Zipf-Mandelbrot
        else if (i == 3) params <- runif(k, 0, 1)              # Bernoulli
        else params <- sample(SDs, k, replace = TRUE)          # Normal distributions
        
        results <- within.subjects.Power(POPULATION_SIZE = POPULATION_SIZE,
                                         N = N,
                                         REPETITIONS = 1, 
                                         TARGET_ARS = TARGET_ARS, 
                                         TESTS = TESTS, 
                                         generateDM = FUNs[[i]], params1 = params, params2 = params, 
                                         VERBOSE = FALSE
        )
        results$Trial <- repetition    # fill in trial information
        results$k <- k                 # fill in the number of features
        results$Model <- Models[[i]]   # fill in the model
        exp <- rbind(exp, results)     # bind results to dataframe
      }
    }
  
  exp$Model <- as.factor(exp$Model)
  return (exp)
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements a simulation procedure to estimate the power of AR statistical tests 
# and within-subjects designs.
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
within.subjects.Power <- function(
  POPULATION_SIZE = 100,        # size of the populations (of referents, not proposals) used to estimate the values of epsilon's for various target AR's
  N = 20,                       # sample size (proposals)
  REPETITIONS = 100,            # number of samples drawn repeatedly from the infinite population
  TARGET_ARS,                   # array of target AR values for the populations of referents that are simulated
  TESTS,                        # array of functions that implement statistical inference tests for comparing AR's
  generateDM, params1, params2, # *generateDM* is a function that generates dissimilarity matrices parameterized by *params1* (for the first population) and *params2* (for the second population)
  VERBOSE = FALSE)              # if TRUE, progress results are printed at the console
{
  # dataframe with results returned by this function
  results <- list()
  
  # run the simulation (total number of trials is REPETITIONS * length(TARGET_ARS))
  for(trial in 1:REPETITIONS)
  {
    epsilon1 <- generatePopulationOfReferentsOfExactAR(POPULATION_SIZE, N, generateDM, params1, sapply(TARGET_ARS, FUN=function(x) return (x[[1]]))) # generate populations of finite size to estimate epsilon's
    epsilon2 <- generatePopulationOfReferentsOfExactAR(POPULATION_SIZE, N, generateDM, params2, sapply(TARGET_ARS, FUN=function(x) return (x[[2]]))) # generate populations of finite size to estimate epsilon's
    
    for(j in 1:length(epsilon1))
    {
      TARGET_AR_1 <- TARGET_ARS[[j]][[1]]
      TARGET_AR_2 <- TARGET_ARS[[j]][[2]]
      
      # sample one group from each population
      A1 <- AM.fromDM(generateDM(N, params1)$DM, epsilon1[[j]])
      A2 <- AM.fromDM(generateDM(N, params2)$DM, epsilon2[[j]])
      
      # run tests
      for(i in 1:length(TESTS))
        results <- rbind(results, list(N, trial, TARGET_AR_1, TARGET_AR_2, AR(A1), AR(A2), TESTS[[i]]$Name, TESTS[[i]]$Fun(A1, A2)$p.value))
      
      if (VERBOSE) cat(sprintf("AR1 = %.3f AR2 = %.3f DELTA = %.3f \r\n", TARGET_AR_1, TARGET_AR_2, abs(TARGET_AR_2 - TARGET_AR_1)))
    }
  }
  
  # create data frame with the results
  results <- as.data.frame(results)
  names(results) <- c("N", "Trial", "TargetAR1", "TargetAR2", "AR1", "AR2", "Test", "p")
  results$N <- as.integer(results$N)
  results$Trial <- as.integer(results$Trial)
  results$TargetAR1 <- as.numeric(results$TargetAR1)
  results$TargetAR2 <- as.numeric(results$TargetAR2)
  results$AR1 <- as.numeric(results$AR1)
  results$AR2 <- as.numeric(results$AR2)
  results$Test <- as.factor(as.character(results$Test))
  results$p <- as.numeric(results$p)
  
  return (results)
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Computes tolerance epsilons for a population of referents with specified 
# mean agreement rates of TARGET_ARs.
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
generatePopulationOfReferentsOfExactAR <- function(POPULATION_SIZE, N, generateDM, params, TARGET_ARs)
{
  # generate dissimilarity matrices
  v <- c()
  for(i in 1:POPULATION_SIZE) 
    v <- c(v, generateDM(N, params)$DM)
  
  # compute the quantiles, using all the dissimilarity values, that corresponds to TARGET_ARs
  v <- sort(v)
  v <- v[c((POPULATION_SIZE * N + 1):length(v))] # remove diagonal zero's
  q <- quantile(v, probs = TARGET_ARs)
  
  return (as.vector(q))
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Runs simulations for exact TARGET_AR and peak NTR1 values
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
simulations.Exact.ARs <- function(
  POPULATION_SIZE = 100,   # size of the population to be generated
  REPETITIONS = 100,       # number of trials
  SEED)                    # random numbers seed for replication purposes
{
  {
    # set seed for replication purposes
    if (!is.null(SEED)) 
    {
      set.seed(SEED)
      cat("Random number generator seed set to", SEED, "\r\n")
    }
    
    SDs <- c(5.42, 2.56, 1.63, 1.15, .88, .69, .576, .493, .416)  # Parameters for the normal distributions (from T. Tsandilas)
    betas <- c(.306, .64, 1.05, 1.56, 2.25, 3.26, 5.0, 8.3, 18.4) # Parameters for the Zipf-Mandelbrot distributions (from T. Tsandilas)
    
    Models <- c("Expert / Zipf-Mandelbrot / Identity",
                "Codebook / Zipf-Mandelbrot / Lin", 
                "Codebook / Bernoulli / Lin", 
                "Computer / Normal / Minkowski (p=0.5)", 
                "Computer / Normal / Minkowski (p=1)", 
                "Computer / Normal / Minkowski (p=2)", 
                "Computer / Normal / Squared Euclidean")
    
    FUNs <- c(generateDM.ZipfMandelbrot.Overlap, 
              generateDM.ZipfMandelbrot.Lin, 
              generateDM.Bernoulli.Lin, 
              generateDM.Minkowski.0.5, 
              generateDM.Minkowski.1, 
              generateDM.Minkowski.2, 
              generateDM.SqrEuclidean)
  }
  
  # dataframe with results returned from this function
  exp <- list()
  
  # run simulations
  for(i in 1:length(Models))
  {
    if (i == 1) Ks <- c(1) else Ks <- c(5, 10)  # number of features for each proposal
    for(k in Ks)
    {
      cat(Models[[i]], "& k =", k, "\r\n")
      
      for(repetition in 1:REPETITIONS)
      {
        cat("\tRepetition", repetition, "of", REPETITIONS, "\r\n")
        
        errorAR <- c()
        peakNTR1 <- NULL
        for(j in 1:9) 
        {
          TARGET_AR <- j * 0.1
          if (i == 1) params <- betas[[j]]                             # Zipf-Mandelbrot and overlap (k=1)
          else if (i == 2) params <- sample(betas, k, replace = TRUE)  # Zipf-Mandelbrot (k=5/10)
          else if (i == 3) params <- runif(k, 0, 1)                    # Bernoulli (k=5/10)
          else params <- sample(SDs, k, replace = TRUE)                # Normal (k=5/10)
          
          DM <- FUNs[[i]](POPULATION_SIZE, params)$DM  # generate a dissimilarity matrix corresponding to a population of proposals of size POPULATION_SIZE
          AM <- AM.fromDM(DM, Epsilon.fromDM.ExactARs(DM, c(ifelse(i==1, 0, TARGET_AR)))[[1]]) # compute the agreement matrix (AM) for this population
          errorAR[[j]] <- abs(TARGET_AR - AR(AM))
          peakNTR1 <- max(peakNTR1, NTR(AM)$NTR1)
        }
        exp <- rbind(exp, list(Models[[i]], k, repetition, mean(errorAR), peakNTR1))
      }
    }
  }
  
  # create data frame from the list of results
  exp <- as.data.frame(exp)
  names(exp) <- c("Model", "k", "Trial", "ARError", "PeakNTR1")
  exp$Model <- as.character(exp$Model)
  exp$k <- as.integer(exp$k)
  exp$Trial <- as.integer(exp$Trial)
  exp$ARError <- as.numeric(exp$ARError)
  exp$PeakNTR1 <- as.numeric(exp$PeakNTR1)
  exp$N <- POPULATION_SIZE
  
  return (exp)
}
