#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements the percentile bootstrap method described in (Wilcox, 2012) [pp.411-412]
# for comparing measures of location for two dependent groups. In our case,
# the two groups are represented by the agreement matrixes A1 and A2.
#
# - FUN specifies the calculation function for the agreement rate, e.g., mean, median, etc.
# - weight is 0.5 for comparing medians; see (Wilcox, 2012) [p.335] and the implementation
#   of the trimpb(..) function, and 1 otherwise.
# - B is the number of boostrap samples
#
# R. Wilcox. (2012). Modern Statistics for the Social and Behavioral Sciences. CRC Press
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
within.subjects.percentile.bootstrap.AR <- function(A1, A2, FUN = mean, weight = 1.0, B = 2000)
{
  N <- nrow(A1)
  Ib <- 0
  C <- 0
  for(trial in 1:B)
  {
    # generate bootstrap sample
    indexes <- sample(1:N, N, replace = TRUE)
    
    # count the number of times the boostrap estimate for group 1 is less than the estimate for group 2
    diff <- FUN(ARs(A1[indexes, indexes])) - FUN(ARs(A2[indexes, indexes]))
    if (diff < 0) Ib <- Ib + 1
    else if (diff == 0) C <- C + 1
  }
  
  p <- (Ib + weight*C)/B
  p <- 2*min(p, 1-p)
  
  return (list(p.value = p))
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements the bootstrap-t method described in (Wilcox, 2012) [pp.268-269]
# for comparing measures of location for two dependent groups. In our case,
# the two groups are represented by the agreement matrixes A1 and A2.
#
# - B is the number of boostrap samples
#
# R. Wilcox. (2012). Modern Statistics for the Social and Behavioral Sciences. CRC Press
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
within.subjects.bootstrap.t.AR <- function(A1, A2, B = 2000)
{
  N <- nrow(A1)
  X <- ARs(A1) - ARs(A2)
  
  # computes the t statistic for two dependent groups
  diff <- mean(X)
  t <- diff/(sd(X)/sqrt(N))
  
  if (any(is.nan(t), is.infinite(t))) # safety check
    return (list(p.value = 0))
  
  # bootstrap the t statistic
  bt <- c()
  for(trial in 1:B)
  {
    # generate the bootstrap sample
    indexes <- sample(1:N, N, replace = TRUE)
    bX <- ARs(A1[indexes,indexes]) - ARs(A2[indexes,indexes])
    
    # compute the t statistic for the bootstrap sample
    bt[[trial]] <- (mean(bX)-diff)/(sd(bX)/sqrt(N))
    if (any(is.nan(bt[[trial]]), is.infinite(bt[[trial]]))) # safety check
      bt[[trial]] <- 0
  }
  
  # compute the 95% CI for the boostrapped t's
  q <- quantile(bt, probs = c(0.025, 0.975))
  
  # reject if the original t value is outside the 95% CI
  p <- ifelse(any(t <= q[[1]], t >= q[[2]]), 0, 1) # not an actual p-value, but useful to automatically compute the Type I error rate in our simulations
  
  return (list(p.value = p))
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements the Vrd test (Vatavu and Wobbrock, 2015) for within-subjects designs
# and two dependent groups represented by the agreement matrixes.
#
# Radu-Daniel Vatavu and Jacob O. Wobbrock. 2015. Formalizing Agreement Analysis for 
# Elicitation Studies: New Measures, Significance Test, and Toolkit. In Proceedings of
# the 33rd Annual ACM Conference on Human Factors in Computing Systems (CHI '15). 
# ACM, New York, NY, USA, 1325-1334. 
# DOI: https://doi.org/10.1145/2702123.2702223
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Vrd.test.2015 <- function(A1, A2)
{
  N <- nrow(A1)
  AR1 <- AR(A1)
  AR2 <- AR(A2)
  test <- (N*(N-1)/2) * (AR1-AR2)^2 / (AR1+AR2-2*CR(A1,A2))
  
  return (list(
    test = test, 
    p = ifelse(test > qchisq(.999, df=1), .001, ifelse(test > qchisq(.99, df=1), .01, ifelse(test > qchisq(.95, df=1), .05, 1))),
    p.value = 1 - pchisq(test, df=1)
  ))
}

#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Implements an experimental variant of the Vrd test (Vatavu and Wobbrock, 2015) 
# for within-subjects designs and two dependent groups represented by their
# agreement matrixes.
#
# In this version, the degrees of freedom (df) are corrected
# from 1 to N/2 according to the intuition that there are (N-1)*(N/2) observations
# and each user forms relations with the other N-1 users.
#
# Note: this version is experimental only, without any mathematical proof.
#++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
Vrd.test <- function(A1, A2)
{
  N <- nrow(A1)
  old.test <- Vrd.test.2015(A1,A2)$test
  
  return (list(
    test = old.test,
    p.value = 1 - pchisq(old.test, df=N/2) # correction for df
  ))
}