#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Sets up the runtime environment: installation and loading of required libraries, constants
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
setupEnvironment <- function(workingDirectory)
{
  setwd(workingDirectory)
  
  if (require(plyr) == FALSE) install.packages("plyr")
  if (require(partitions) == FALSE) install.packages("partitions")    # integer partitions
  if (require(zipfR) == FALSE) install.packages("zipfR")              # Zipf-Mandelbrot distribution
  if (require(fossil) == FALSE) install.packages("fossil")            # testing the triangle inequality for distances
  if (require(philentropy) == FALSE) install.packages("philentropy")  # Minkowski distance
  if (require(testthat) == FALSE) install.packages("testthat")        # unit tests
  if (require(DescTools) == FALSE) install.packages("DescTools")      # formatting functions for real numbers    
  
  library(plyr)
  library(partitions) 
  library(zipfR)
  library(fossil)
  library(philentropy)
  library(testthat)
  library(DescTools)
  
  source("Rallfun-v37.txt")          # Rand Wilcox's library of R functions for robust statistics; https://dornsife.usc.edu/labs/rwilcox/software/
  source("EUES/Measures.R")          # measures of agreement for end-user elicitation studies
  source("EUES/WithinSubjects.R")    # tests for within-subjects designs for end-user elicitation studies
  source("EUES/BetweenSubjects.R")   # tests for between-subjects designs for end-user elicitation studies
  source("EUES/Simulations.DissimilarityMatrix.R")       
  source("EUES/Simulations.TypeIError.Power.R")      
  source("EUES/UnitTests.R")         # unit tests
}

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Main
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
setupEnvironment("C:/R code/") # replace with actual folder name

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Unit tests
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
runAllUnitTests()

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Run simulations for Type I error rates and Power for statistical tests. 
# Takes about 24 hours on a 3.6GHz i7 CPU and produces four csv files.
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
runTypeIandPowerSimulations(POPULATION_SIZE=100, N=20, REPETITIONS=100, SEED=1000) 

wsTESTS <- c("t-test", "Wilcoxon", "Vrd2015", "Vrd", "pb-m", "bt")
bsTESTS <- c("t-test", "Welch's t-test", "Mann-Whitney", "Cliff", "BMP", "KS", "Vb2016-CM", "Vb2016-nCM", "pb-m", "bt")

exp <- read.csv("data/within-subjects-typeIerror.csv", header = TRUE)
LatexTable.Summary(exp, wsTESTS); LatexTable.Detailed(exp, wsTESTS, report05 = TRUE)

exp <- read.csv("data/within-subjects-power.csv", header = TRUE)
LatexTable.Summary(exp, wsTESTS); LatexTable.Detailed(exp, wsTESTS, report05 = TRUE)

exp <- read.csv("data/between-subjects-power.csv", header = TRUE)
LatexTable.Summary(exp, bsTESTS); LatexTable.Detailed(exp, bsTESTS, report05 = TRUE)

exp <- read.csv("data/between-subjects-typeIerror.csv", header = TRUE)
LatexTable.Summary(exp, bsTESTS); LatexTable.Detailed(exp, bsTESTS, report05 = TRUE)
ddply(exp, ~TargetAR, function(dataframe) {
  N <- nrow(dataframe)
  return (c(
    N = N, 
    AR1mean = sprintf("%.3f", mean(dataframe$AR1)), 
    AR1sd = sprintf("%.3f", sd(dataframe$AR1)), 
    AR2mean = sprintf("%.3f", mean(dataframe$AR2)), 
    AR2sd = sprintf("%.3f", sd(dataframe$AR2))))
})

#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
# Run simulations for exact AR rates
# Takes about 3 hours on a 3.6GHz i7 CPU.
#+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
write.csv(simulations.Exact.ARs(POPULATION_SIZE = 100, REPETITIONS = 100, SEED = 1000), file = "ExactARs.csv")
exp <- read.csv("data/ExactARs.csv")
ddply(exp, ~Model+k, function(dataframe){
  return (c(
    ARError = sprintf("%.3f (%.3f)", mean(dataframe$ARError), sd(dataframe$ARError)), 
    peakNTR1 = sprintf("%.3f (%.3f)", mean(dataframe$PeakNTR1), sd(dataframe$PeakNTR1))
  ))
})
